@@ -318,27 +318,13 @@ where
         public_inputs: &[PublicInputValue],
         transcript_init: &'static [u8],
     ) -> Result<(), Error> {
-        let gates = verifier_data.key().padded_gates();
-        let pi_indexes = verifier_data.public_inputs_indexes();
-
-        let mut dense_pi = vec![BlsScalar::zero(); gates];
-
-        public_inputs
-            .iter()
-            .map(|pi| pi.0.clone())
-            .flatten()
-            .zip(pi_indexes.iter().cloned())
-            .for_each(|(value, pos)| {
-                dense_pi[pos] = -value;
-            });
-
-        let mut verifier = Verifier::new(transcript_init);
-
-        verifier.verifier_key.replace(*verifier_data.key());
-
-        let opening_key = pub_params.opening_key();
-
-        verifier.verify(proof, opening_key, &dense_pi)
+        verify(
+            pub_params,
+            verifier_data,
+            proof,
+            public_inputs,
+            transcript_init,
+        )
     }
 
     /// Return the list of public inputs generated by the gadget
@@ -347,3 +333,34 @@ where
     /// Returns the Circuit size padded to the next power of two.
     fn padded_gates(&self) -> usize;
 }
+
+/// Verify the provided proof for the compiled verifier data
+pub fn verify(
+    pub_params: &PublicParameters,
+    verifier_data: &VerifierData,
+    proof: &Proof,
+    public_inputs: &[PublicInputValue],
+    transcript_init: &'static [u8],
+) -> Result<(), Error> {
+    let gates = verifier_data.key().padded_gates();
+    let pi_indexes = verifier_data.public_inputs_indexes();
+
+    let mut dense_pi = vec![BlsScalar::zero(); gates];
+
+    public_inputs
+        .iter()
+        .map(|pi| pi.0.clone())
+        .flatten()
+        .zip(pi_indexes.iter().cloned())
+        .for_each(|(value, pos)| {
+            dense_pi[pos] = -value;
+        });
+
+    let mut verifier = Verifier::new(transcript_init);
+
+    verifier.verifier_key.replace(*verifier_data.key());
+
+    let opening_key = pub_params.opening_key();
+
+    verifier.verify(proof, opening_key, &dense_pi)
+}
