@@ -12,22 +12,12 @@
 #[cfg(feature = "alloc")]
 pub use crate::{
     circuit::{self, Circuit, PublicInputValue, VerifierData},
-    commitment_scheme::kzg10::{
-        key::{CommitKey, OpeningKey},
-        PublicParameters,
-    },
-    constraint_system::{Point, StandardComposer, Variable},
+    commitment_scheme::{CommitKey, OpeningKey, PublicParameters},
+    constraint_system::{Constraint, TurboComposer, Witness, WitnessPoint},
     proof_system::{Prover, ProverKey, Verifier},
 };
 
-pub use crate::proof_system::{Proof, VerifierKey};
-
-/// Re-exported [`dusk-bls12_381::BlsScalar`].
-pub use dusk_bls12_381::BlsScalar;
-
-/// Re-exported [`dusk-jubjub::JubJubScalar`] &
-/// [`dusk-jubjub::JubJubAffine`].
-pub use dusk_jubjub::{JubJubAffine, JubJubScalar};
-
-/// Collection of errors that the library exposes/uses.
 pub use crate::error::Error;
+pub use crate::proof_system::{Proof, VerifierKey};
+pub use dusk_bls12_381::BlsScalar;
+pub use dusk_jubjub::{JubJubAffine, JubJubExtended, JubJubScalar};
